import {Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {DeleteResult, Repository} from 'typeorm';
import {Album} from './album.entity';
import {CreateAlbumDto} from './dto/create-album.dto';
import {UpdateAlbumDto} from './dto/update-album.dto';
import {UUID} from "crypto";

@Injectable()
export class AlbumService {
  constructor(
    @InjectRepository(Album)
    private albumRepository: Repository<Album>,
  ) {}

  findAll(): Promise<Album[] | string | null> {
    return this.albumRepository
      .find({
        order: {
          artist: 'ASC',
          title: 'ASC',
        },
      })
      .then((albums) => {
        return albums;
      })
      .catch((error) => {
        return `There was a problem getting the list of albums: ${error}`;
      });
  }

  findOneById(uuid: UUID): Promise<Album | string | null> {
    return this.albumRepository
      .findOne({
        where: {
          uuid: uuid,
        },
        relations: {
          songs: true,
        },
        order: {
          songs: {
            trackNumber: 'ASC',
          },
        },
      })
      .then((album) => {
        return album;
      })
      .catch((error) => {
        return `There was a problem creating the Album identified by ID ${uuid}: ${error}`;
      })
      .finally(() => {
        return `There was a problem creating the Album identified by ID ${uuid}`;
      });
  }

  async create(createAlbumDto: CreateAlbumDto): Promise<Album | string | null> {
    const album = this.albumRepository.create({
      title: createAlbumDto.title,
      artist: createAlbumDto.artist,
      genre: createAlbumDto.genre,
    });
    return await this.albumRepository
      .save(album)
      .then((savedAlbum) => {
        return savedAlbum;
      })
      .catch((error) => {
        return `There was a problem creating the Album (${createAlbumDto.title} by ${createAlbumDto.artist} (${createAlbumDto.genre})): ${error}`;
      });
  }

  async update(
    uuid: string,
    updateAlbumDto: UpdateAlbumDto,
  ): Promise<Album | string | null> {
    if (uuid == updateAlbumDto.uuid) {
      const albumToUpdate = await this.albumRepository.findOneBy({
        uuid: updateAlbumDto.uuid,
      });
      if (!albumToUpdate) {
        console.error("AlbumService: update: Didn't find album: ", uuid);
        return null;
      }

      await this.albumRepository.update(
        { uuid: updateAlbumDto.uuid },
        {
          title: updateAlbumDto.title,
          artist: updateAlbumDto.artist,
          genre: updateAlbumDto.genre,
        },
      );
      return await this.albumRepository.findOneBy({
        uuid: updateAlbumDto.uuid,
      });
    } else {
      console.error(
        'AlbumService: update: IDs do not match',
        uuid,
        updateAlbumDto,
      );
      return null;
    }
  }

  async remove(uuid: UUID): Promise<DeleteResult | string | null> {
    return await this.albumRepository
      .delete(uuid)
      .then((deleteResult) => {
        return deleteResult;
      })
      .catch((error) => {
        return `There was a problem deleting the Album identified by ID ${uuid}: ${error}`;
      });
  }
}
