import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { DeleteResult, Repository } from 'typeorm';
import { Album } from '../album/album.entity';
import { Song } from './song.entity';
import { CreateSongDto } from './dto/create-song.dto';
import { UpdateSongDto } from './dto/update-song.dto';
import {UUID} from "crypto";

@Injectable()
export class SongService {
  constructor(
    @InjectRepository(Song)
    private songRepository: Repository<Song>,
    @InjectRepository(Album)
    private albumRepository: Repository<Album>,
  ) {}

  findAll(): Promise<Song[] | string | null> {
    return this.songRepository
      .find({
        order: {
          trackNumber: 'ASC',
        },
      })
      .then((songs) => {
        return songs;
      })
      .catch((error) => {
        return `There was a problem getting the list of songs: ${error}`;
      });
  }

  findOneById(uuid: UUID): Promise<Song | string | null> {
    return this.songRepository
      .findOneBy({ uuid: uuid })
      .then((albums) => {
        return albums;
      })
      .catch((error) => {
        return `There was a problem getting the song identified by ID ${uuid}: ${error}`;
      });
  }

  async create(createSongDto: CreateSongDto): Promise<Song | string | null> {
    const album = await this.albumRepository.findOneBy({
      uuid: createSongDto.albumId,
    });
    if (album) {
      const song = new Song();
      song.title = createSongDto.title;
      song.duration = createSongDto.duration;
      song.trackNumber = createSongDto.trackNumber;
      song.album = album;
      return this.songRepository
        .save(song)
        .then((albums) => {
          return albums;
        })
        .catch((error) => {
          return `There was a problem creating the song (${createSongDto.trackNumber} ${createSongDto.title} (${createSongDto.duration}s) on the Album ${album.title} by ${album.artist}): ${error}`;
        });
    } else {
      throw new Error(`Unable to find Album with ID ${createSongDto.albumId}`);
    }
  }

  async update(
    uuid: UUID,
    updateSongDto: UpdateSongDto,
  ): Promise<Song | string | null> {
    if (uuid == updateSongDto.uuid) {
      const album = await this.albumRepository.findOneBy({
        uuid: updateSongDto.albumId,
      });
      const songToUpdate = await this.songRepository.findOneBy({
        uuid: updateSongDto.uuid,
      });
      if (!songToUpdate || !album) {
        console.error('SongService: update: Song or Album not found');
        return null;
      }

      await this.songRepository.update(
        { uuid: updateSongDto.uuid },
        {
          title: updateSongDto.title,
          duration: updateSongDto.duration,
          trackNumber: updateSongDto.trackNumber,
          album: album,
        },
      );
      const song = await this.songRepository.findOneBy({
        uuid: updateSongDto.uuid,
      });
      return song;
    } else {
      console.error('SongService: update: IDs do not match');
      return null;
    }
  }

  async remove(uuid: UUID): Promise<DeleteResult | string | null> {
    return await this.songRepository
      .delete(uuid)
      .then((deleteResult) => {
        return deleteResult;
      })
      .catch((error) => {
        return `There was a problem deleting the Song identified by ID ${uuid}: ${error}`;
      });
  }
}
